#include <stdio.h>
#include <assert.h>
#include "sonyserial.h"

#include <time.h>
#include <sys/time.h>

timecode_t get_timecode() // {{{
{
  struct timeval tv;
  gettimeofday(&tv,NULL);
  time_t tim=tv.tv_sec;
  struct tm *tm=localtime(&tim);
  timecode_t tc={
    .hour = tm->tm_hour,
    .minute = tm->tm_min,
    .second = tm->tm_sec,
    .frame = tv.tv_usec/1000*25/1000
  };
  return tc;
}
// }}}

typedef enum { SS_PLAY=0x01, SS_RECORD=0x02, SS_FF=0x04, SS_REW=0x08,
               SS_EJECT=0x10, SS_STOP=0x20, SS_STANDBY=0x80 } sstatus_t;
void set_status(sonyserial_t *ser,sstatus_t status) // {{{
{
  unsigned char *st=sonyserial_getstatus(ser);
  if (status==SS_RECORD) {
    status|=SS_PLAY;
  }
  st[1]=status;
}
// }}}

void handle_event(sonyserial_t *ser,sonyevent_t *evt) // {{{
{
  if (evt->type==SEV_TC_PLAY) {
    set_status(ser,SS_PLAY);
  } else if (evt->type==SEV_TC_RECORD) {
    set_status(ser,SS_RECORD);
  } else if (evt->type==SEV_TC_FF) {
    set_status(ser,SS_FF);
  } else if (evt->type==SEV_TC_REV) {
    set_status(ser,SS_REW);
//  } else if (evt->type==SEV_TC_EJECT) {
  } else if (evt->type==SEV_TC_STOP) {
    set_status(ser,SS_STOP);
  }
}
// }}}


int main()
{
  sonyserial_t *ser=open_sonyserial("/dev/ttyUSB0");
  if (!ser) {
    return 1;
  }

  sonyserial_settimecode_cb(ser,get_timecode);

//  unsigned char *st=sonyserial_getstatus(ser);
//  st[1]=0x01;

  while (1) {
#if 1
    sonyserial_readone(ser);
    sonyevent_t evt=sonyserial_getevent(ser);
    if (evt.type!=SEV_NONE) {
      dump_event(&evt);
      handle_event(ser,&evt);
    }
#else
    int ch=sonyserial_readone(ser);
    if (ch>=0) {
      printf("%02x ",ch); fflush(stdout);
    }
#endif
  }

  close_sonyserial(ser);

  return 0;
}
