#ifndef _SONYSERIAL_H
#define _SONYSERIAL_H

#include <stdbool.h>

typedef struct _sonyserial_t sonyserial_t;

// (will be converted to BCD on-wire format)
typedef struct { unsigned char hour,minute,second,frame; } timecode_t;

typedef struct {
  enum { SEV_NONE=-1,
         SEV_SC_LOCAL=0x0000, // enable?
         SEV_TC_STOP=0x0200, SEV_TC_PLAY, SEV_TC_RECORD,
         SEV_TC_STANDBY=0x0204, // on?
         SEV_TC_EJECT=0x020F,
         SEV_TC_FF=0x0210,
         SEV_TC_JOGFF, SEV_TC_VARFF, SEV_TC_SHUTTLEFF, // tape_speed
         SEV_TC_JOGPAUSE=-2,SEV_TC_SHUTTLEPAUSE=-3, // (synthesized from TC_{JOG,SHUTTLE}FF
         SEV_TC_REV=0x0220,
         SEV_TC_JOGREV, SEV_TC_VARREV, SEV_TC_SHUTTLEREV, // tape_speed
         SEV_TC_PREROLL=0x0230, // [as specified in preroll preset]
         SEV_TC_CUETO, // timecode
//         SEV_TC_SYNC=0x0234,
         SEV_TC_FULLEE=0x0260, // on?
         SEV_TC_SELECTEE=0x063, // (always on)
         SEV_TC_EDIT=0x0264, // on?

         SEV_PS_EDITPRESET=0x0430, // preset;
         SEV_PS_COLORFRAME=0x0435, // mode
         SEV_PS_TIMERMODE=0x0436, // mode

         SEV_PS_AUTO=0x0440, SEV_PS_SPOTERASE=0x0442, SEV_PS_AUDIOSPLIT=0x0444 // on?
  } type;
  union {
    bool enable; // on/off // SEV_LOCAL
    double tape_speed; // multiplicator of normal play speed
    timecode_t tc;
    unsigned char mode;
    unsigned char preset[2];
  };
} sonyevent_t;

sonyserial_t *open_sonyserial(const char *device); // NULL on error
void close_sonyserial(sonyserial_t *ser);

void sonyserial_setstatus(sonyserial_t *ser,unsigned char *status);
unsigned char *sonyserial_getstatus(sonyserial_t *ser);

// read next character
int sonyserial_readone(sonyserial_t *ser);

//void dump_block(unsigned char *buf);

bool sonyserial_hasevent(sonyserial_t *ser);
sonyevent_t sonyserial_getevent(sonyserial_t *ser);

void dump_event(sonyevent_t *evt);

typedef timecode_t (*TIMECODE_CB)();
void sonyserial_settimecode_cb(sonyserial_t *ser,TIMECODE_CB cb);

#endif
